#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  7 15:26:40 2021

@author: ricardofaria

regrid roms and ww3 to wrf grid

"""


import numpy as np
from netCDF4 import Dataset
import glob
from grid_interp import interpolate_2d_grid


wrf_path = sorted(glob.glob('*wrf.nc'))
roms_path = sorted(glob.glob('*roms.nc'))

roms_grid = Dataset(roms_path[0])
lat_roms = roms_grid.variables['Latitude'][:]
lon_roms = roms_grid.variables['Longitude'][:]

mask = roms_grid.variables['salt'][0,0,:]
mask = np.ma.masked_where(mask == 1e+37, mask)
mask = np.ma.getmask(mask)

roms_grid.close()


for f in wrf_path:
    print('Opening file ', f)
    wrf_grid = Dataset(f, 'r+')
    
    # regrid WRF grid to roms grid
    lat_wrf = wrf_grid.variables['Latitude'][:]
    lon_wrf = wrf_grid.variables['Longitude'][:]
    
    
    wrf_list_vars = wrf_grid.variables.keys()
    
    wrf_grid.createDimension('lon_rgd', len(lon_roms))
    wrf_grid.createDimension('lat_rgd', len(lat_roms))
    nc_longitude = wrf_grid.createVariable('Longitude_regrid',  np.float32, 'lon_rgd')
    nc_latitude = wrf_grid.createVariable('Latitude_regrid',  np.float32, 'lat_rgd')
    nc_longitude.units = 'degree_east'
    nc_latitude.units = 'degree_north'
    nc_longitude[:] = lon_roms.data
    nc_latitude[:] = lat_roms.data
    
    for v in list(wrf_list_vars):
        
        if 'Longitude' == v or 'Latitude' == v or 'height' == v or 'Time' == v or 'Longitude_regrid' == v or 'Latitude_regrid' == v :
            print('Skip ', v)
        else :
            print('Work on ', v)
            # interpolate wrf data from old wrf grid to the new roms grid
            var = wrf_grid.variables[v][:]
            # var_str = v+'_rgd'
            if len(var.shape) == 2:
                nc_file_final_var = wrf_grid.createVariable(v+'_rgd', np.float32, (wrf_grid.variables[v].dimensions[0]+'_rgd', wrf_grid.variables[v].dimensions[1]+'_rgd'), zlib=True)
                nc_file_final_var[:] = interpolate_2d_grid(var, mask, lon_wrf, lat_wrf, lon_roms, lat_roms)
            elif len(var.shape) == 3:
                nc_file_final_var = wrf_grid.createVariable(v+'_rgd', np.float32, (wrf_grid.variables[v].dimensions[0], wrf_grid.variables[v].dimensions[1]+'_rgd', wrf_grid.variables[v].dimensions[2]+'_rgd'), zlib=True)
                for d1 in range(var.shape[0]):
                    nc_file_final_var[d1, :] = interpolate_2d_grid(var[d1, :], mask, lon_wrf, lat_wrf, lon_roms, lat_roms)
            elif len(var.shape) == 4:
                nc_file_final_var = wrf_grid.createVariable(v+'_rgd', np.float32, (wrf_grid.variables[v].dimensions[0], wrf_grid.variables[v].dimensions[1], wrf_grid.variables[v].dimensions[2]+'_rgd', wrf_grid.variables[v].dimensions[3]+'_rgd'), zlib=True)
                for d1 in range(var.shape[0]):
                    for d2 in range(var.shape[1]):
                        nc_file_final_var[d1, d2, :] = interpolate_2d_grid(var[d1, d2, :], mask, lon_wrf, lat_wrf, lon_roms, lat_roms)
            
    wrf_grid.close()
